@@ -1,9 +1,9 @@
 #ifndef lint
-static const char rcsid[] =
-    "@(#) $Id: cupdate.c 42 2021-09-06 17:02:09Z leres $ (LBL)";
+static const char rcsid[] __attribute__ ((unused)) =
+    "@(#) $Id: cupdate.c 47 2022-08-06 21:55:16Z leres $ (LBL)";
 #endif
 /*
- * Copyright (c) 2005, 2006, 2008, 2009, 2010, 2013, 2016, 2018, 2020, 2021
+ * Copyright (c) 2005, 2006, 2008, 2009, 2010, 2013, 2016, 2018, 2020, 2021, 2022
  *	Craig Leres
  */
 
@@ -14,11 +14,14 @@ static const char rcsid[] =
 #include <ctype.h>
 #include <curses.h>
 #include <errno.h>
+#include <signal.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
+#include "setsignal.h"
+
 #define CTRL(c) ((c) & 0x1F)
 
 /* Globals */
@@ -26,13 +29,22 @@ const char *prog;
 int debug;
 int verbose;
 int waitsecs = 1;
+int waiting;
+int done;
+WINDOW *win;
+#ifdef SIGWINCH
+sig_t oldsigwinch;
+#endif
 extern const char version[];
 
-
 /* Forwards */
 char *buildcommand(int, char **);
 void __attribute__((noreturn, format (printf, 1, 2))) die(const char *, ...);
 void finish(void);
+void sigint(int);
+#ifdef SIGWINCH
+void sigwinch(int);
+#endif
 char *runcommand(char *);
 int trimws(char *, int);
 void __attribute__((noreturn)) usage(void);
@@ -45,7 +57,6 @@ main(int argc, char **argv)
 	char *cp, *ep, *cmd, *out, ch;
 	fd_set rfds;
 	struct timeval tv;
-	WINDOW *win;
 	const char *tty;
 	struct stat sbuf;
 
@@ -99,6 +110,11 @@ main(int argc, char **argv)
 	    chmod(tty, sbuf.st_mode & ~S_IWGRP) < 0)
 		tty = NULL;
 
+	(void)setsignal(SIGINT, sigint);
+#ifdef SIGWINCH
+	oldsigwinch = setsignal(SIGWINCH, sigwinch);
+#endif
+
 	cbreak();
 	noecho();
 	clear();
@@ -113,7 +129,14 @@ main(int argc, char **argv)
 		FD_ZERO(&rfds);
 		FD_SET(fileno(stdin), &rfds);
 		tv.tv_sec = waitsecs;
+		++waiting;
 		n = select(fileno(stdin) + 1, &rfds, NULL, NULL, &tv);
+		waiting = 0;
+
+		/* ^C exit */
+		if (done)
+			break;
+
 		if (n < 0) {
 			if (errno == EINTR)
 				continue;
@@ -180,6 +203,19 @@ buildcommand(int ac, char **av)
 	return (cmd);
 }
 
+void
+die(const char *fmt, ...)
+{
+	va_list ap;
+
+	finish();
+	va_start(ap, fmt);
+	fprintf(stderr, "%s: ", prog);
+	vfprintf(stderr, fmt, ap);
+	va_end(ap);
+	exit(1);
+}
+
 void
 finish()
 {
@@ -237,18 +273,25 @@ runcommand(char *cmd)
 }
 
 void
-die(const char *fmt, ...)
+sigint(int signo)
 {
-	va_list ap;
-
-	finish();
-	va_start(ap, fmt);
-	fprintf(stderr, "%s: ", prog);
-	vfprintf(stderr, fmt, ap);
-	va_end(ap);
-	exit(1);
+	++done;
+	return;
 }
 
+#ifdef SIGWINCH
+void
+sigwinch(int signo)
+{
+	(void)oldsigwinch(signo);
+	if (waiting && win != NULL) {
+		redrawwin(win);
+		refresh();
+	}
+	return;
+}
+#endif
+
 /* Strip whitespace at the end of each line preserving the \n */
 int
 trimws(char *str, int cc)
